﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using System.IO.Ports;
using System.IO;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.Windows;

namespace AudibleChatClient
{
    public class Listen
    {
        // Chris
        UdpClient sender;
        UdpClient listener;
        State sSend;
        State sRecv;
        int _port;
        IPAddress _ip;
        keepAlive _keepAlive;

        public bool waiting;

        public delegate int messageReceiveCallback(byte[] bytes);

        Dictionary<int, List<messageReceiveCallback>> messageCallbacks;

        bool _isListening;

        // Chris
        public Listen()
        {
            try
            {
                messageCallbacks = new Dictionary<int, List<messageReceiveCallback>>();

                waiting = false;
                _isListening = false;

                addCallback(Protocol.SC_KEEPALIVE, onKeepaliveAck);
            }
            catch (Exception e)
            {
                Console.WriteLine("Listen constructor failed: " + e.Message);
            }
        }

        ~Listen()
        {
            Console.WriteLine("DTOR");
            stopListening();
            removeCallback(Protocol.SC_KEEPALIVE, onKeepaliveAck);
        }

        public void initConnection(int port, IPAddress ip)
        {
            try
            {
                _port = port;
                _ip = ip;

                sender = new UdpClient();

                sSend = new State(sender);
                sSend.ipEndPoint = new IPEndPoint(_ip, 3210);

                listener = new UdpClient();
                sRecv = new State(listener);
                sRecv.ipEndPoint = new IPEndPoint(_ip, _port);
            }
            catch (Exception e)
            {
                Console.WriteLine("Listen init failed: " + e.Message);
                MessageBox.Show("Could not connect to the provided IP Address and Port");
            }
        }

        // Chris
        public bool startListening()
        {
            try
            {
                listener.Client.Bind(new IPEndPoint(IPAddress.Any, sRecv.ipEndPoint.Port));
                listener.BeginReceive(new AsyncCallback(receiveCallback), sRecv);
                _isListening = true;
                beginKeepalive();
                return true;
            }
            catch (Exception e)
            {
                Console.WriteLine("begin listen failed: " + e.Message);
                MessageBox.Show("Could not connect to the provided IP Address and Port");
                return false;
            }
        }

        // Chris
        public void stopListening()
        {
            try
            {
                if (listener != null)
                {
                    listener.Close();
                    waiting = false;
                    _isListening = false;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("stop listen failed: " + e.Message);
            }
        }

        // Chris
        public void sendMessage(byte[] msg)
        {
            try
            {
                Console.WriteLine("     sending " + msg[1]);
                sender.Send(msg, msg.Length, sSend.ipEndPoint);
            }
            catch (Exception e)
            {

                Console.WriteLine("failed to send message: " + e.Message);
            }
        }

        // Chris
        private void receiveCallback(IAsyncResult ar)
        {
            try
            {
                State state = (State)(ar.AsyncState);
                byte[] received = listener.EndReceive(ar, ref sRecv.ipEndPoint);

                if (_isListening)
                {
                    listener.BeginReceive(receiveCallback, sRecv);

                    if (received[0] == 0x01)
                    {
                        if (messageCallbacks.Keys.Contains(received[1]))
                        {
                            Console.WriteLine("     received " + received[1]);

                            foreach (messageReceiveCallback d in messageCallbacks[received[1]])
                            {
                                byte[] reply = new byte[2];
                                reply[0] = 0x01;
                                reply[1] = Convert.ToByte(d(received));
                            }
                        }
                        else
                        {
                            Console.WriteLine("No callback assigned to receive code " + received[1]);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("receive callback failed: " + e.Message);
            }
        }

        // Chris
        public void beginKeepalive()
        {
            if (_keepAlive == null)
            {
                _keepAlive = new keepAlive(sender);
            }
        }

        // Chris
        private int onKeepaliveAck(byte[] bytes)
        {
            _keepAlive.keep = true;

            return Protocol.CS_SUCCESS;
        }

        // Chris
        public bool isListening
        {
            get { return _isListening; }
        }

        // Chris
        public IPAddress serverIP
        {
            get { return _ip; }
        }

        // Chris
        public void addCallback(int messageCode, messageReceiveCallback callback)
        {
            try
            {
                if (messageCallbacks.Keys.Contains(messageCode))
                {
                    if (messageCallbacks[messageCode] == null)
                    {
                        messageCallbacks[messageCode] = new List<messageReceiveCallback>();
                    }
                }
                else
                {
                    messageCallbacks.Add(messageCode, new List<messageReceiveCallback>());
                }

                if (!messageCallbacks[messageCode].Contains(callback))
                {
                    messageCallbacks[messageCode].Add(callback);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("failed to add callback: " + e.Message);
            }
        }

        // Chris
        public void removeCallback(int messageCode, messageReceiveCallback callback)
        {
            try
            {
                if (messageCallbacks.ContainsKey(messageCode) && messageCallbacks[messageCode].Contains(callback))
                {
                    messageCallbacks[messageCode].Remove(callback);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("failed to remove callback: " + e.Message);
            }
        }
    }
}
