﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using DataBaseTest;

namespace AudibleChatClient
{

    public partial class Login : Form
    {
        public String userName = "";
        public String passWord = "";
        Encoding asciiEncoder = Encoding.ASCII;
        public byte[] credentials = new byte[32];
        public byte[] recv = new byte[1];
        byte[] cnctToSvr = new byte[34];

        public delegate void onClientMessageReceive(string msg);
        
        //form constructor
        public Login()
        {
            InitializeComponent();

            connect.listener.addCallback(Protocol.SC_USERLOGIN_SUCCESS, onLoginSuccess);
            connect.listener.addCallback(Protocol.SC_USERLOGIN_FAIL, onLoginFail);
            connect.listener.addCallback(Protocol.SC_USERLOGOUT_SUCCESS, onLogoutSuccess);
            connect.listener.addCallback(Protocol.SC_USERLOGOUT_FAIL, onLogoutFail);
        }

        #region Login functions

        private void btnConnect_Click(object sender, EventArgs e)
        {
            _login();
        }

        private void _login_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
            {
                _login();
                e.Handled = true;
            }
        }

        public void _login()
        {
            try
            {
                userName = txtUN.Text;

                List<byte> msg = new List<byte>();
                msg.Add(0x01);
                msg.Add(Protocol.CS_USERLOGIN);

                byte[] usr = new byte[Protocol.SIZE_USERNAME];
                ASCIIEncoding.ASCII.GetBytes(txtUN.Text).ToList().CopyTo(usr);
                msg.AddRange(usr.ToList());

                byte[] pw = new byte[Protocol.SIZE_PASSWORD_HASH];
                ASCIIEncoding.ASCII.GetBytes(PassHash.HashPass(txtPW.Text)).ToList().CopyTo(pw);
                msg.AddRange(pw.ToList());

                connect.listener.sendMessage(msg.ToArray());
            }
            catch
            {
            }
        }

        public void getCreds()
        {
            //convert username to bytes
            userName = txtUN.Text;
            byte[] UNbytes = new byte[asciiEncoder.GetByteCount(userName)];
            UNbytes = asciiEncoder.GetBytes(userName);
            UNbytes.CopyTo(credentials, 0);

            //convert password to bytes
            passWord = txtPW.Text;
            byte[] PWbytes = new byte[asciiEncoder.GetByteCount(passWord)];
            PWbytes = asciiEncoder.GetBytes(passWord);
            PWbytes.CopyTo(credentials, 16);
        }

        public void getConnectionPacket()
        {
            //add SOH request to connect to server and username & password to byte array
            //to be sent to the server.
                    
            cnctToSvr[0] = 0x01;
            cnctToSvr[1] = 0x05;
            credentials.CopyTo(cnctToSvr, 2);
        }
        
        // Chris
        private int onLoginSuccess(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onLoginSuccess);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                this.Hide();
                PublicRoom pRoom = new PublicRoom(this);
                pRoom.ShowDialog();
                return Protocol.CS_SUCCESS;
            }
        }

        // Chris
        private int onLoginFail(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onLoginFail);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                userName = "";
                MessageBox.Show("Login failed");

                return Protocol.CS_SUCCESS;
            }
        }
        

        #endregion

        #region Register

        private void btnCreate_Click(object sender, EventArgs e)
        {
            CreateUser newUser = new CreateUser(this);
            newUser.ShowDialog();
            txtUN.Text = userName;
            txtPW.Text = passWord;
        }

        #endregion

        private void Login_FormClosed(object sender, FormClosedEventArgs e)
        {
            byte[] msg = new byte[2];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_USERLOGOUT;

            connect.listener.sendMessage(msg);

            connect.listener.removeCallback(Protocol.SC_USERLOGIN_SUCCESS, onLoginSuccess);
            connect.listener.removeCallback(Protocol.SC_USERLOGIN_FAIL, onLoginFail);
            connect.listener.removeCallback(Protocol.SC_USERLOGOUT_SUCCESS, onLogoutSuccess);
            connect.listener.removeCallback(Protocol.SC_USERLOGOUT_FAIL, onLogoutFail);

            Application.Exit();
        }
        // Chris
        private int onLogoutSuccess(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onLogoutSuccess);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                Application.Exit();
                return Protocol.CS_SUCCESS;
            }
        }

        // Chris
        private int onLogoutFail(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onLogoutFail);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                Application.Exit();
                return Protocol.CS_SUCCESS;
            }
        }

        private void Login_Load(object sender, EventArgs e)
        {

        }

       
    }
}
