﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AudibleChatClient
{
    public partial class ManageUsersAdd : Form
    {
        Encoding asciiEncoder = Encoding.ASCII;
        PrivateRoom parent;

        public ManageUsersAdd(PrivateRoom _p)
        {
            InitializeComponent();
            InitializeListBox();

            parent = _p;

            connect.listener.addCallback(Protocol.SC_ADDMEMBER_SUCCESS, onAddMemberSuccess);
            connect.listener.addCallback(Protocol.SC_ADDMEMBER_FAIL, onAddMemberFail);
            connect.listener.addCallback(Protocol.SC_USERLIST, onUsersRequested);
        }

        public void InitializeListBox()
        {
            byte[] _mesg = new byte[3];
            _mesg[0] = 0x01;
            _mesg[1] = Protocol.CS_USERLISTREQUEST;
            _mesg[2] = 0x03;
            connect.listener.sendMessage(_mesg);
        }

        private int onUsersRequested(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onUsersRequested);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                if (bytes[2] == 0x03)
                {
                    String name = "";

                    for (int i = 4; i < bytes.Length; i += (int)Protocol.SIZE_USERNAME)
                    {
                        name = asciiEncoder.GetString(bytes, i, (int)Protocol.SIZE_USERNAME);

                        if (name != "" && !lstUsers.Items.Contains(name))
                        {
                            name = name.Remove(name.IndexOf('\0'));
                            lstUsers.Items.Add(name);
                        }
                    }

                    return Protocol.CS_SUCCESS;
                }
                return Protocol.CS_FAILURE;
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            List<byte> msg = new List<byte>();
            msg.Add(0x01);
            msg.Add(Protocol.CS_ADDMEMBER);

            byte[] room = new byte[Protocol.SIZE_ROOMNAME];
            ASCIIEncoding.ASCII.GetBytes(parent.Text).ToList().CopyTo(room);
            msg.AddRange(room.ToList());

            msg.Add((byte)lstUsers.CheckedItems.Count);


            for (int i = 0; i < lstUsers.CheckedItems.Count; i++)
            {
                byte[] user = new byte[Protocol.SIZE_USERNAME];
                ASCIIEncoding.ASCII.GetBytes(lstUsers.CheckedItems[i].ToString()).ToList().CopyTo(user);
                msg.AddRange(user.ToList());
            }

            connect.listener.sendMessage(msg.ToArray());
        }

        private int onAddMemberSuccess(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onAddMemberSuccess);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                MessageBox.Show("Successfully added " + bytes[2] + " member(s).");
                this.Close();
                return Protocol.CS_SUCCESS;
            }
        }

        private int onAddMemberFail(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onAddMemberFail);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                MessageBox.Show("Error adding member(s)");
                return Protocol.CS_SUCCESS;
            }
        }

        private void ManageUsers_FormClosed(object sender, FormClosedEventArgs e)
        {
            connect.listener.removeCallback(Protocol.SC_ADDMEMBER_SUCCESS, onAddMemberSuccess);
            connect.listener.removeCallback(Protocol.SC_ADDMEMBER_FAIL, onAddMemberFail);
            connect.listener.removeCallback(Protocol.SC_USERLIST, onUsersRequested);
        }



    }
}
