﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using AwesomeAudio;

namespace AudibleChatClient
{
    public partial class PrivateRoom : Form
    {
#region definitions

        Encoding asciiEncoder = Encoding.ASCII;
        Login login;
        PublicRoom parent;
        public Keys key;

        bool hasToken;
        bool isNewPress;
        bool isKeyDown;

        UDPSender audioSender;
        List<string> connectedIPs;

        public static Color _formColor;
        public static Color _windowColor;
        public static Color _fontColor;
        public static Font _fontSize;
        public static Font _font;

#endregion

        public PrivateRoom(String roomName, Login login, PublicRoom parent)
        {
            InitializeComponent();

            this.KeyPreview = true;
            this.parent = parent;
            this.login = login;

            hasToken = false;
            isNewPress = true;
            isKeyDown = false;

            this.Text = roomName;

            label1.Text = "0/127";

            connect.listener.addCallback(Protocol.SC_USERTXT, onTxtReceive);
            connect.listener.addCallback(Protocol.SC_USERLIST, onUsersUpdated);
            connect.listener.addCallback(Protocol.SC_TOKENREQUEST_SUCCESS, onTokReqSuccess);
            connect.listener.addCallback(Protocol.SC_TOKENREQUEST_FAIL, onTokReqFail);
            connect.listener.addCallback(Protocol.SC_TOKENHOLD_SUCCESS, onTokHoldSuccess);
            connect.listener.addCallback(Protocol.SC_TOKENHOLD_FAIL, onTokHoldFail);
            connect.listener.addCallback(Protocol.SC_ROOMLIST, onRoomListUpdate);
            connect.listener.addCallback(Protocol.SC_USERJOIN, onUserJoin);
            connect.listener.addCallback(Protocol.SC_USERLEAVE, onUserLeave);
            connect.listener.addCallback(Protocol.SC_AUDIOBEGIN, onAudioBegin);
            connect.listener.addCallback(Protocol.SC_AUDIOEND, onAudioEnd);

            reqUsers();
            reqRooms();

            audioSender = new UDPSender();
            connectedIPs = new List<string>();
        }

#region Form Functions

        private void reqUsers()
        {
            byte[] msg = new byte[3];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_USERLISTREQUEST;
            msg[2] = 0x02; //key that the list of users is for the current private room

            connect.listener.sendMessage(msg);
        }

        private void _send_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
            {
                txtUserSend.Clear();
            }
        }

        private void reqRooms()
        {
            byte[] msg = new byte[2];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_ROOMLISTREQUEST;

            connect.listener.sendMessage(msg);
        }

        private void _send()
        {

            if (txtUserSend.Text.Count() < 128)
            {
                String message = "";

                if (txtUserSend.Text != "")
                {
                    message = login.userName + ": " + txtUserSend.Text + "\n";
                    sendMessage(message);
                    txtUserSend.Clear();
                    txtUserSend.Focus();
                }
            }
            else
            {
                MessageBox.Show("You have entered too many characters to transmit. Please change the length of you message to 127 characters.");
            }
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            _send();
        }

        public void sendMessage(String message)
        {
            if (message.Length > Protocol.SIZE_USERTXT)
            {
                message = new string(message.ToCharArray(), 0, (int)Protocol.SIZE_USERTXT);
            }

            List<byte> msg = new List<byte>();
            msg.Add(0x01);
            msg.Add(Protocol.CS_USERTXT);
            msg.Add(0x01);

            byte[] txt = new byte[Protocol.SIZE_USERTXT];
            ASCIIEncoding.ASCII.GetBytes(message).ToList().CopyTo(txt);
            msg.AddRange(txt.ToList());

            connect.listener.sendMessage(msg.ToArray());
        }

        public void updateText(string message)
        {
            this.txtRoomText.AppendText(message + "\n");
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetUpKeyStroke keys = new SetUpKeyStroke(this);
            keys.ShowDialog();

        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if ((Keys)e.KeyValue == key)
            {
                e.SuppressKeyPress = true; // do this to 'eat' the keystroke
            }
            else
            {
                e.SuppressKeyPress = false;
            }

            base.OnKeyDown(e);
        }

        private void PrivateRoom_KeyDown(object sender, KeyEventArgs e)
        {
            if ((Keys)e.KeyValue == key)
            {
                isKeyDown = true;

                if (isNewPress && !hasToken)
                {
                    byte[] tokMsg = { 0x01, Protocol.CS_TOKENREQUEST };
                    connect.listener.sendMessage(tokMsg);
                }
                else if (!isNewPress && hasToken)
                {
                    byte[] tokMsg = { 0x01, Protocol.CS_TOKENHOLD };
                    connect.listener.sendMessage(tokMsg);
                }


                isNewPress = false;
            }
        }

        private void PrivateRoom_KeyUp(object sender, KeyEventArgs e)
        {
            if ((Keys)e.KeyValue == key)
            {
                if (hasToken)
                {
                    if (isKeyDown)
                    {
                        byte[] msg = { 0x01, Protocol.CS_TOKENRELEASE };
                        connect.listener.sendMessage(msg);
                    }

                    lblToken.Visible = false;
                    picToken.Visible = false;

                    Console.WriteLine("         STOPPING");
                    audioSender.StopMicrophoneSend();

                    hasToken = false;

                    isKeyDown = false;
                }

                isNewPress = true;
            }
        }

        private void txtUserSend_TextChanged(object sender, EventArgs e)
        {
            label1.Text = txtUserSend.Text.Count().ToString() + "/127";
        }

#endregion


        public void refreshWindow()
        {
            //txtRoomText
            txtRoomText.ForeColor = _fontColor;
            txtRoomText.BackColor = _windowColor;
            txtRoomText.Font = _font;
            txtRoomText.Font = _fontSize;
            txtRoomText.Refresh();
            txtRoomText.Update();

            toolStrip1.BackColor = _formColor;

            //txtUserSend
            txtUserSend.ForeColor = _fontColor;
            txtUserSend.BackColor = _windowColor;
            txtUserSend.Font = _font;
            txtUserSend.Font = _fontSize;
            txtUserSend.Refresh();
            txtUserSend.Update();

            toolStrip1.BackColor = _formColor;
            tsFile.BackColor = _formColor;
            if (tsFile.HasDropDownItems)
            {
                foreach (ToolStripDropDownItem item in tsFile.DropDownItems)
                {
                    item.BackColor = _formColor;
                }
            }

            this.BackColor = _formColor;

            this.Update();
        }
#region callbacks

        public int onTxtReceive(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onTxtReceive);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                if (bytes[2] == 0x01)
                {
                    string message = asciiEncoder.GetString(bytes, 3, (int)Protocol.SIZE_USERTXT);
                    this.updateText(message);

                    return Protocol.CS_SUCCESS;
                }

                return Protocol.CS_FAILURE;
            }
        }

        private int onUsersUpdated(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onUsersUpdated);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                if (bytes[2] == 0x02)
                {
                    String name = "";

                    lstUsers.Items.Clear();

                    for (int i = 4; i < bytes.Length; i += (int)Protocol.SIZE_USERNAME)
                    {
                        name = asciiEncoder.GetString(bytes, i, (int)Protocol.SIZE_USERNAME);

                        if (name != "")
                        {
                            name = name.Remove(name.IndexOf('\0'));
                            lstUsers.Items.Add(name);
                        }
                    }

                    return Protocol.CS_SUCCESS;
                }

                return Protocol.CS_FAILURE;
            }
        }

        private int onRoomListUpdate(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onRoomListUpdate);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                String name = "";

                lstRooms.Items.Clear();

                for (int i = 3; i < bytes.Length; i += (int)Protocol.SIZE_ROOMNAME)
                {
                    name = asciiEncoder.GetString(bytes, i, (int)Protocol.SIZE_USERNAME);

                    if (name != "")
                    {
                        name = name.Remove(name.IndexOf('\0'));
                        lstRooms.Items.Add(name);
                    }
                }

                return Protocol.CS_SUCCESS;
            }
        }

        private int onTokReqSuccess(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onTokReqSuccess);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                picToken.Visible = true;
                lblToken.Visible = true;

                hasToken = true;

                List<string> receivingIPs = new List<string>();

                for (int i = 3; i < bytes.Length; i += (int)Protocol.SIZE_IPTXT)
                {
                    string ipStr = asciiEncoder.GetString(bytes, i, (int)Protocol.SIZE_IPTXT).TrimEnd('\0');
                    IPAddress ip;

                    if (IPAddress.TryParse(ipStr, out ip))
                    {
                        if (ip != null)
                        {
                            Console.WriteLine("audio to: " + ip.ToString());
                            receivingIPs.Add(ip.ToString());
                        }
                    }
                }

                if (receivingIPs.Count > 0)
                {
                    Console.WriteLine("         STARTING");
                    audioSender.StartMicrophoneSend(receivingIPs.ToArray());
                }

                return Protocol.CS_SUCCESS;
            }
        }

        private int onTokReqFail(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onTokReqFail);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                lblToken.Visible = false;
                picToken.Visible = false;

                Console.WriteLine("         STOPPING");
                audioSender.StopMicrophoneSend();

                return Protocol.CS_SUCCESS;
            }
        }

        private int onTokHoldSuccess(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onTokHoldSuccess);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                return Protocol.CS_SUCCESS;
            }
        }

        private int onTokHoldFail(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onTokHoldFail);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                lblToken.Visible = false;
                picToken.Visible = false;

                hasToken = false;

                Console.WriteLine("         STOPPING");
                audioSender.StopMicrophoneSend();

                return Protocol.CS_SUCCESS;
            }
        }

        private int onUserJoin(byte[] bytes)
        {
            byte[] msg = { 0x01, Protocol.CS_USERLISTREQUEST, 0x02 };
            connect.listener.sendMessage(msg);

            return Protocol.CS_SUCCESS;
        }

        private int onUserLeave(byte[] bytes)
        {
            byte[] msg = { 0x01, Protocol.CS_USERLISTREQUEST, 0x02 };
            connect.listener.sendMessage(msg);

            return Protocol.CS_SUCCESS;
        }

        private int onAudioBegin(byte[] bytes)
        {
            return Protocol.CS_SUCCESS;
        }

        private int onAudioEnd(byte[] bytes)
        {
            return Protocol.CS_SUCCESS;
        }

#endregion

        private void PrivateRoom_FormClosed(object sender, FormClosedEventArgs e)
        {
            byte[] msg = { 0x01, Protocol.CS_USERLEAVEREQUEST };
            connect.listener.sendMessage(msg);

            connect.listener.removeCallback(Protocol.SC_USERTXT, onTxtReceive);
            connect.listener.removeCallback(Protocol.SC_USERLIST, onUsersUpdated);
            connect.listener.removeCallback(Protocol.SC_TOKENREQUEST_SUCCESS, onTokReqSuccess);
            connect.listener.removeCallback(Protocol.SC_TOKENREQUEST_FAIL, onTokReqFail);
            connect.listener.removeCallback(Protocol.SC_TOKENHOLD_SUCCESS, onTokHoldSuccess);
            connect.listener.removeCallback(Protocol.SC_TOKENHOLD_FAIL, onTokHoldFail);
            connect.listener.removeCallback(Protocol.SC_ROOMLIST, onRoomListUpdate);
            connect.listener.removeCallback(Protocol.SC_USERJOIN, onUserJoin);
            connect.listener.removeCallback(Protocol.SC_USERLEAVE, onUserLeave);
            connect.listener.removeCallback(Protocol.SC_AUDIOBEGIN, onAudioBegin);
            connect.listener.removeCallback(Protocol.SC_AUDIOEND, onAudioEnd);
        }

        private void txtUserSend_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                _send();
            }
        }

        private void PrivateRoom_Load(object sender, EventArgs e)
        {
            txtUserSend.Focus();
        }

        private void addMembersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageUsersAdd mUsers = new ManageUsersAdd(this);
            mUsers.ShowDialog();
        }

        private void removeMembersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageUsersRemove mUsers = new ManageUsersRemove(this);
            mUsers.ShowDialog();
        }

        private void splitter1_SplitterMoved(object sender, SplitterEventArgs e)
        {

        }

        private void settingsToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            SetUpKeyStroke setKey = new SetUpKeyStroke(this);
            setKey.Show();
        }

        private void tsFile_Click(object sender, EventArgs e)
        {

        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            this.Refresh();
        }

        private void btnSend_Click_1(object sender, EventArgs e)
        {
            _send();
        }

    }
}
