﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using AwesomeAudio;

namespace AudibleChatClient
{
    public partial class PublicRoom : Form
    {
        Encoding asciiEncoder = Encoding.ASCII;
        Login parent;
        public String roomName = "";
        PrivateRoom privateRoom;
        public AudioPlayer audioPlayer;

        public static Color _formColor;
        public static Color _windowColor;
        public static Color _fontColor;
        public static Font _fontSize;
        public static Font _font;

        public PublicRoom(Login parent)
        {
            InitializeComponent();
            this.parent = parent;
            
            lstRooms.MouseDown += new MouseEventHandler(lstRooms_MouseDown);
            //privateRoom = null;
            label1.Text = "0/127";
            connect.listener.addCallback(Protocol.SC_ROOMLIST, onRoomListUpdate);
            connect.listener.addCallback(Protocol.SC_USERTXT, onTxtReceive);
            //connect.listener.addCallback(Protocol.SC_USERLOGOUT_SUCCESS, onLogoutSuccess);
            connect.listener.addCallback(Protocol.SC_USERLIST, onUsersUpdated);
            connect.listener.addCallback(Protocol.SC_USERJOINREQUEST_SUCCESS, onJoiningRoomSuccess);
            connect.listener.addCallback(Protocol.SC_USERJOINREQUEST_FAIL, onJoiningRoomFail);

            reqUsers();
            reqRooms();
            this.txtUserSend.Focus();

            audioPlayer = new AudioPlayer();
        }

        public void refreshUserSettings()
        {
            //txtRoomText
            txtRoomText.ForeColor = _fontColor;
            txtRoomText.BackColor = _windowColor;
            txtRoomText.Font = _font;
            txtRoomText.Font = _fontSize;
            txtRoomText.Refresh();
            txtRoomText.Update();

            toolStrip1.BackColor = _formColor;

            tsFile.BackColor = _formColor;
            if (tsFile.HasDropDownItems)
            {
                foreach (ToolStripDropDownItem item in tsFile.DropDownItems)
                {
                    item.BackColor = _formColor;
                }
            }

            tsWindow.BackColor = _formColor;
            if (tsWindow.HasDropDownItems)
            {
                foreach (ToolStripDropDownItem item in tsWindow.DropDownItems)
                {
                    item.BackColor = _formColor;
                }
            }



            //txtUserSend
            txtUserSend.ForeColor = _fontColor;
            txtUserSend.BackColor = _windowColor;
            txtUserSend.Font = _font;
            txtUserSend.Font = _fontSize;
            txtUserSend.Refresh();
            txtUserSend.Update();

            this.BackColor = _formColor;
            this.Update();


            this.Invoke((MethodInvoker)delegate
            {
                try
                {
                    if (privateRoom != null)
                    {
                        privateRoom.refreshWindow();
                    }
                }
                catch { }
            });
        }

        private void _send_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
            {
                txtUserSend.Clear();
            }
        }

        private void _send()
        {
            if (txtUserSend.Text.Count() < 128)
            {
                if (txtUserSend.Text != "")
                {
                    sendMessage(parent.userName + ": " + txtUserSend.Text);
                    txtUserSend.Focus();
                    txtUserSend.Clear();
                }
            }
            else
            {
                MessageBox.Show("You have entered too many characters to transmit. Please change the length of you message to 127 characters.");
            }
        }

        private void reqUsers()
        {
            byte[] msg = new byte[3];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_USERLISTREQUEST;
            msg[2] = 0x01;

            connect.listener.sendMessage(msg);
        }

        private void reqRooms()
        {
            byte[] msg = new byte[2];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_ROOMLISTREQUEST;

            connect.listener.sendMessage(msg);

        }

        public void updateText(string message)
        {
            txtRoomText.AppendText(message);
            txtRoomText.AppendText("\n");
        }

        public void sendMessage(String message)
        {
            if (message.Length > Protocol.SIZE_USERTXT)
            {
                message = new string(message.ToCharArray(), 0, (int)Protocol.SIZE_USERTXT);
            }

            List<byte> msg = new List<byte>();
            msg.Add(0x01);
            msg.Add(Protocol.CS_USERTXT);
            msg.Add(0x00);

            byte[] txt = new byte[Protocol.SIZE_USERTXT];
            ASCIIEncoding.ASCII.GetBytes(message).ToList().CopyTo(txt);
            msg.AddRange(txt.ToList());

            connect.listener.sendMessage(msg.ToArray());
            txtUserSend.Focus();
        }

        public void joinRoom(string roomName)
        {
            //privateRoom = new PrivateRoom(roomName, parent);
            //privateRoom.Show();
            privateRoom = new PrivateRoom(roomName, parent, this);
            privateRoom.Show();
            refreshUserSettings();
        }

#region Events

        private void PublicRoom_FormClosed(object sender, FormClosedEventArgs e)
        {
            //byte[] msg = new byte[2];
            //msg[0] = 0x01;
            //msg[1] = Protocol.CS_USERLOGOUT;

            //connect.listener.sendMessage(msg);

            connect.listener.removeCallback(Protocol.SC_ROOMLIST, onRoomListUpdate);
            connect.listener.removeCallback(Protocol.SC_USERTXT, onTxtReceive);
            //connect.listener.removeCallback(Protocol.SC_USERLOGOUT_SUCCESS, onLogoutSuccess);
            connect.listener.removeCallback(Protocol.SC_USERLIST, onUsersUpdated);
            connect.listener.removeCallback(Protocol.SC_USERJOINREQUEST_SUCCESS, onJoiningRoomSuccess);
            connect.listener.removeCallback(Protocol.SC_USERJOINREQUEST_FAIL, onJoiningRoomFail);

            //Environment.Exit(0);
            Application.Exit();
            //connect.endProgram();
            Console.WriteLine("HERE");
        }

        private void tsSettings_Click(object sender, EventArgs e)
        {
            UserWindowSettings uwSet = new UserWindowSettings(this, _font, _fontColor, _fontSize, _windowColor, _formColor);
            uwSet.ShowDialog();
            refreshUserSettings();
        }

        void lstRooms_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ContextMenu cm = new ContextMenu();
                MenuItem mi = new MenuItem("Join Room");
                mi.Click += new System.EventHandler(mi_Join_Click);
                cm.MenuItems.Add(mi);
                System.Drawing.Point p = new Point(e.X, e.Y);
                lstRooms.SelectedIndex = lstRooms.IndexFromPoint(e.X, e.Y);
                cm.Show(lstRooms, p);
            }
        }

        private void mi_Join_Click(object sender, EventArgs e)
        {
            if (lstRooms.SelectedItem != null)
            {
                string r = lstRooms.SelectedItem.ToString();

                List<byte> msg = new List<byte>();
                msg.Add(0x01);
                msg.Add(Protocol.CS_USERJOINREQUEST);

                byte[] room = new byte[Protocol.SIZE_ROOMNAME];
                ASCIIEncoding.ASCII.GetBytes(r).ToList().CopyTo(room);
                msg.AddRange(room.ToList());

                connect.listener.sendMessage(msg.ToArray());

                roomName = r;
            }
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            _send();
        }
        
        private void tsCreateRoom_Click(object sender, EventArgs e)
        {
            new RoomSpecs().ShowDialog();
        }

        private void tsLogOut_Click(object sender, EventArgs e)
        {
            byte[] msg = new byte[2];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_USERLOGOUT;

            connect.listener.sendMessage(msg);
        }
        
        private void updateUsersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            byte[] msg = new byte[3];
            msg[0] = 0x01;
            msg[1] = Protocol.CS_USERLISTREQUEST;
            msg[2] = 0x01;

            connect.listener.sendMessage(msg);
        }

        private void txtUserSend_TextChanged(object sender, EventArgs e)
        {
            label1.Text = txtUserSend.Text.Count().ToString() + "/127";
        }

        private void txtUserSend_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                _send();
            }
            txtUserSend.Focus();
        }

        private void updateRoomsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            reqRooms();
            txtUserSend.Focus();
        }

#endregion

#region Callbacks

        private int onTxtReceive(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onTxtReceive);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                if (bytes[2] != 0x01)
                {
                    string message = asciiEncoder.GetString(bytes, 3, (int)Protocol.SIZE_USERTXT);
                    updateText(message);
                    txtUserSend.Focus();
                    return Protocol.CS_SUCCESS;
                }

                return Protocol.CS_FAILURE;
            }
        }

        private int onUsersUpdated(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onUsersUpdated);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                if (bytes[2] == 0x01)
                {
                    String name = "";

                    lstUsers.Items.Clear();

                    for (int i = 4; i < bytes.Length; i += (int)Protocol.SIZE_USERNAME)
                    {
                        name = asciiEncoder.GetString(bytes, i, (int)Protocol.SIZE_USERNAME);

                        if (name != "")
                        {
                            name = name.Remove(name.IndexOf('\0'));
                            lstUsers.Items.Add(name);
                            txtUserSend.Focus();
                        }
                    }

                    return Protocol.CS_SUCCESS;
                }

                return Protocol.CS_FAILURE;
            }
        }

        private int onRoomListUpdate(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onRoomListUpdate);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                String name = "";

                lstRooms.Items.Clear();

                for (int i = 3; i < bytes.Length; i += (int)Protocol.SIZE_ROOMNAME)
                {
                    name = asciiEncoder.GetString(bytes, i, (int)Protocol.SIZE_ROOMNAME);

                    if (name != "")
                    {
                        name = name.Remove(name.IndexOf('\0'));
                        lstRooms.Items.Add(name);
                        //txtUserSend.Focus();
                    }
                }

                return Protocol.CS_SUCCESS;
            }
        }

        //private int onLogoutSuccess(byte[] bytes)
        //{
        //    connect.restartProgram();
        //    return Protocol.CS_SUCCESS;
        //}

        public int onJoiningRoomSuccess(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onJoiningRoomSuccess);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                joinRoom(roomName);
                return Protocol.CS_SUCCESS;
            }
        }

        private int onJoiningRoomFail(byte[] bytes)
        {
            if (InvokeRequired)
            {
                Listen.messageReceiveCallback d = new Listen.messageReceiveCallback(onJoiningRoomFail);
                this.Invoke(d, new object[] { bytes });

                return Protocol.CS_FAILURE;
            }
            else
            {
                MessageBox.Show("Failed to join room: " + roomName);
                return Protocol.CS_SUCCESS;
            }

        }

#endregion

        private void PublicRoom_Load(object sender, EventArgs e)
        {
            txtUserSend.Focus();
        }
    }
}
