﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AudibleChatClient
{
    public partial class UserWindowSettings : Form
    {
        PublicRoom _pPub;
        PrivateRoom _pPriv;
        bool pub = false;
        bool priv = false;
        FontConverter cvt = new FontConverter();
        Font font, fontSize;
        Color fontColor, formColor, windowColor;
        bool defaults = false;

        public UserWindowSettings(PublicRoom parent, Font _font, Color _fontColor, Font _fontSize, Color _windowColor, Color _formColor)
        {
            InitializeComponent();
            InitComboBoxes();
            _pPub = parent;
            pub = true;
            font = _font;
            fontSize = _fontSize;
            fontColor = _fontColor;
            formColor = _formColor;
            windowColor = _windowColor;
        }

        public UserWindowSettings(PrivateRoom parent)
        {
            InitializeComponent();
            InitComboBoxes();
            _pPriv = parent;
            priv = true;
        }

        public void InitComboBoxes()
        {
            initFontBox();
            initFontSizeBox();
            initFontColorBox();
            initWindowColor();
            initFormColor();
        }

        public void initFontBox()
        {
            cmbFont.DataSource = FontFamily.Families.Where(p => p.IsStyleAvailable(FontStyle.Regular)).ToList();
            cmbFont.DisplayMember = "Name";


            //bool first = true;
            //foreach (FontFamily font in System.Drawing.FontFamily.Families)
            //{
                
            //    if (first)
            //    {
            //        cmbFont.Text = font.Name;
            //        first = false;
            //    }
            //    cmbFont.Items.Add(font.Name);
            //}
           
        }

        public void initFontSizeBox()
        {
            cmbFontSize.Text = "8";
            for (int i = 6; i < 100; i++)
            {
                cmbFontSize.Items.Add(i);
            }
        }

        public void initFontColorBox()
        {
            bool first = true;
            foreach (KnownColor kcolor in Enum.GetValues(typeof(KnownColor)))
            {
                Color color = Color.FromKnownColor(kcolor);
                
                
                if (!color.IsSystemColor)
                {
                    cmbFontColor.Items.Add(color);
                    if (first)
                    {
                        cmbFontColor.Text = "Black";
                        first = false;
                    }
                }
            }
        }

        public void initWindowColor()
        {
            bool first = true;
            foreach (KnownColor kcolor in Enum.GetValues(typeof(KnownColor)))
            {
                Color color = Color.FromKnownColor(kcolor);
                
                if (!color.IsSystemColor)
                {
                    cmbWindowColor.Items.Add(color);
                    if (first)
                    {
                        cmbWindowColor.Text = color.Name;
                        first = false;
                    }
                }
            }
        }

        public void initFormColor()
        {
            bool first = true;
            foreach (KnownColor kcolor in Enum.GetValues(typeof(KnownColor)))
            {
                
                Color color = Color.FromKnownColor(kcolor);
                
                if (!color.IsSystemColor)
                {
                    if (first)
                    {
                        cmbFormColor.Text = color.Name;
                        first = false;
                    }
                    cmbFormColor.Items.Add(color);
                    
                }
            }
        }

        private void UserWindowSettings_Load(object sender, EventArgs e)
        {

        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                PublicRoom._font = cvt.ConvertFromString(cmbFont.Text) as Font;
                PrivateRoom._font = cvt.ConvertFromString(cmbFont.Text) as Font;
            }
            catch
            {
                PublicRoom._font = cvt.ConvertFromString("Arial") as Font;
                PrivateRoom._font = cvt.ConvertFromString("Arial") as Font;
            }
            PublicRoom._fontColor = Color.FromName(cmbFontColor.Text);
            PrivateRoom._fontColor = Color.FromName(cmbFontColor.Text);
            try
            {
                PublicRoom._fontSize = new Font(cmbFont.Text, Convert.ToInt16(cmbFontSize.Text));
                PrivateRoom._fontSize = new Font(cmbFont.Text, Convert.ToInt16(cmbFontSize.Text));
            }
            catch
            {
                PublicRoom._fontSize = new Font(cmbFont.Text, Convert.ToInt16(cmbFontSize.Text));
                PrivateRoom._fontSize = new Font(cmbFont.Text, Convert.ToInt16(cmbFontSize.Text));
  
            }
            if (defaults)
            {
                PublicRoom._formColor = SystemColors.Control;
                PrivateRoom._formColor = SystemColors.Control;
                PublicRoom._windowColor = SystemColors.Window;
                PrivateRoom._windowColor = SystemColors.Window;
            }
            else
            {
                PublicRoom._formColor = Color.FromName(cmbFormColor.Text);
                PrivateRoom._formColor = Color.FromName(cmbFormColor.Text);
                PublicRoom._windowColor = Color.FromName(cmbWindowColor.Text);
                PrivateRoom._windowColor = Color.FromName(cmbWindowColor.Text);
            }
            
            this.Close();
        }

        #region example label
        private void cmbFormColor_SelectedIndexChanged(object sender, EventArgs e)
        {
            defaults = false;
            try
            {
                this.BackColor = Color.FromName(cmbFormColor.Text);
                this.Refresh();
            }
            catch
            {
            }
        }

        private void cmbWindowColor_SelectedIndexChanged(object sender, EventArgs e)
        {
            defaults = false;
            label1.BackColor = Color.FromName(cmbWindowColor.Text);
        }

        private void cmbFontColor_SelectedIndexChanged(object sender, EventArgs e)
        {
            defaults = false;
            label1.ForeColor = Color.FromName(cmbFontColor.Text);
        }

        private void cmbFontSize_SelectedIndexChanged(object sender, EventArgs e)
        {
            defaults = false;
            label1.Font = new Font(cmbFont.Text, Convert.ToInt16(cmbFontSize.Text));
        }

        private void cmbFont_SelectedIndexChanged(object sender, EventArgs e)
        {
            defaults = false;
            try
            {
                label1.Font = cvt.ConvertFromString(cmbFont.Text) as Font;
            }
            catch
            {

            }
        }
        #endregion

        private void btnDefaults_Click(object sender, EventArgs e)
        {
            cmbFont.Text = "Microsoft Sans Serif";
            cmbFontSize.Text = "8";
            cmbFontColor.Text = "Black";
            cmbWindowColor.Text = "Transparent";
            cmbFormColor.Text = "Transparent";
            this.BackColor = SystemColors.Control;
            
            defaults = true;
        }
    }
}
