﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using DataBaseTest;

namespace AudioChatServer
{
    public class Chatroom
    {
        string _owner;
        Dictionary<string, bool> _members;
        string _name;
        public string userWithToken;

        /// <summary>
        /// Create a new chat room to be added to the database. The passed user is added to the
        /// members list at the end of this constructor.
        /// </summary>
        /// <param name="owner">The User object of the user that is creating the chat room.</param>
        /// <param name="name">Name of the chat room name.</param>
        public Chatroom(string owner, string name = "")
        {
            _owner = owner;
            _name = (name == "") ? owner + "'s Chatroom" : name;
            _members = new Dictionary<string, bool>();
            userWithToken = "";

            addMember(owner);
        }

        public bool addMember(string member)
        {
            try
            {
                if (!_members.Keys.Contains(member))
                {
                    _members.Add(member, false);
                    return true;
                }

                return false;
            }
            catch (Exception e)
            {
                Console.WriteLine("Add member failed: " + e.Message);
                return false;
            }
        }

        public bool removeMember(string member)
        {
            try
            {
                if (_members.Keys.Contains(member))
                {
                    _members.Remove(member);
                    return true;
                }

                return false;
            }
            catch (Exception e)
            {
                Console.WriteLine("Remove member failed: " + e.Message);
                return false;
            }
        }

        public bool userJoin(string user, bool checkMembership = true)
        {
            if ((isUserAMember(user) && !isUserInside(user)))// || !checkMembership)
            {
                _members[user] = true;
                return true;
            }

            return false;
        }

        public bool userLeave(string user)
        {
            if (isUserInside(user))
            {
                _members[user] = false;

                foreach (string u in _members.Keys)
                {
                    if (_members[u])
                    {
                        return true;
                    }
                }

                userWithToken = "";

                return true;
            }

            return false;
        }

        public bool isUserAMember(string user)
        {
            return (user != null && _members.Keys.Contains(user));
        }

        public bool isUserInside(string user)
        {
            if (isUserAMember(user))
            {
                return _members[user];
            }

            return false;
        }

        public string roomName
        {
            get { return _name; }
        }

        public string owner
        {
            get { return _owner; }
        }

        public List<string> members
        {
            get { return _members.Keys.ToList(); }
        }

        public List<string> activeMembers
        {
            get
            {
                List<string> result = new List<string>();

                foreach (string u in _members.Keys)
                {
                    if (_members[u])
                    {
                        result.Add(u);
                    }
                }

                return result;
            }
        }

        public override string ToString()
        {
            return _name;
        }
    }
}
