﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MySql.Data.MySqlClient;
using AudioChatServer;

//Going to use UUID for key values

namespace DataBaseTest
{
    public class DatabaseInterface
    {
        public bool Active = false;
        string connectionString;

        MySqlConnection conn = null;
        MySqlDataReader rdr = null;

        public DatabaseInterface()
        {
            connectionString = @"server=localhost;userid=Moog;password=asdlkjpoi;database=chatty";
        }
        public DatabaseInterface(string cs)
        {
            connectionString = cs;
        }
        public DatabaseInterface(string server, string userid, string password, string db)
        {
            connectionString = @"server=" + server + @";userid=" + userid + @";password=" + password + @";database=" + db;
        }

        public bool OpenConnection()
        {
            try
            {
                conn = new MySqlConnection(connectionString);

                conn.Open();
                Active = true;
                return true;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("Error: {0}", ex.ToString());
                CloseConnection();
                Active = false;
                return false;
            }
            //return false;
        }
        public void CloseConnection()
        {
            if (rdr != null)
            {
                rdr.Close();
            }

            if (conn != null)
            {
                conn.Close();
            }

            Active = false;
        }

        #region UserJank
        /// <summary>
        /// Takes in a user name and password, hashes the password, and puts the data into the database.
        /// </summary>
        /// <param name="user">The username string.</param>
        /// <param name="pword">The hashed password string.</param>
        /// <returns>True if successful, false if fail.</returns>
        public bool CreateUser(string user, string pword)
        {
            MySqlTransaction tr = null;
            uint u_id = 0;
            uint p_id = 0;
            try
            {
                tr = conn.BeginTransaction();

                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.Transaction = tr;

                cmd.CommandText = "INSERT INTO Users(u_name) VALUES(@Name)";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@Name", user);
                cmd.ExecuteNonQuery();

                cmd.CommandText = "SELECT LAST_INSERT_ID()";
                rdr = cmd.ExecuteReader();
                rdr.Read();
                u_id = rdr.GetUInt32(0);
                rdr.Close();

                cmd.CommandText = "INSERT INTO Pwrd(phash) VALUES(@phash)";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@phash", pword);
                cmd.ExecuteNonQuery();

                cmd.CommandText = "SELECT LAST_INSERT_ID()";
                rdr = cmd.ExecuteReader();
                rdr.Read();
                p_id = rdr.GetUInt32(0);
                rdr.Close();

                cmd.CommandText = "INSERT INTO U_P(u_id, p_id) VALUES(@u_id, @p_id)";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@u_id", u_id);
                cmd.Parameters.AddWithValue("@p_id", p_id);
                cmd.ExecuteNonQuery();

                tr.Commit();
                rdr = null;
                return true;
            }
            catch (MySqlException ex)
            {
                try
                {
                    tr.Rollback();

                }
                catch (MySqlException ex1) { Console.WriteLine("failed to rollback: " + ex1.Message); }
                if (rdr != null)
                {
                    rdr.Close();
                    rdr = null;
                }
                Console.WriteLine("failure in creating user: " + ex.Message);
                return false;
            }
        }

        /// <summary>
        /// Checks to see if a user is already in the database.
        /// </summary>
        /// <param name="user">A string with the user's username.</param>
        /// <returns>1 is there is user with this name in the database, 0 if not, and -1 if error</returns>
        public int CheckUserExists(string user)
        {
            try
            {
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT EXISTS(SELECT u_name FROM Users WHERE u_name = @Name)";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@Name", user);
                rdr = cmd.ExecuteReader();
                int result = -1;
                if (rdr.Read())
                {
                    result = rdr.GetInt32(0);
                }
                rdr.Close();
                rdr = null;


                return result;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in checking user: " + ex.Message);
                if (rdr != null) rdr.Close();
                return -1;
            }
        }


        public int CheckPassword(string user, string pword)
        {
            try
            {
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT EXISTS(SELECT DISTINCT Users.u_name, Pwrd.phash FROM U_P JOIN Users on U_P.u_id = Users.u_id JOIN Pwrd on U_P.p_id = Pwrd.p_id WHERE Users.u_name = @name AND Pwrd.phash = @pword)";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@Name", user);
                cmd.Parameters.AddWithValue("@pword", pword);
                rdr = cmd.ExecuteReader();

                int result = -1;
                if (rdr.Read())
                {
                    result = rdr.GetInt32(0);
                }
                rdr.Close();
                rdr = null;
                return result;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in checking password: " + ex.Message);
                if (rdr != null)
                {
                    rdr.Close();
                    rdr = null;
                }
                return -1;
            }
        }

        public int ChangePassword(string user, string pword)
        {
            return -1;
        }

        public List<User> GetUserList()
        {
            try
            {
                List<User> users = new List<User>();
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT u_name FROM Users";
                rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    users.Add(new User(rdr.GetString(0)));
                }
                rdr.Close();
                rdr = null;


                return users;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in getting users: " + ex.Message);
                if (rdr != null) rdr.Close();
                return null;
            }
        }
        #endregion

        public Chatroom CreateRoom(string roomName, string owner)
        {
            MySqlTransaction tr = null;
            try
            {
                uint r_id;

                tr = conn.BeginTransaction();

                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.Transaction = tr;

                cmd.CommandText = "INSERT INTO Rooms(r_name, u_id) VALUES(@rame, (SELECT u_id FROM Users WHERE Users.u_name = @owns))";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@rame", roomName);
                cmd.Parameters.AddWithValue("@owns", owner);
                cmd.ExecuteNonQuery();

                cmd.CommandText = "SELECT LAST_INSERT_ID()";
                rdr = cmd.ExecuteReader();
                rdr.Read();
                r_id = rdr.GetUInt32(0);
                rdr.Close();

                cmd.CommandText = "INSERT INTO R_U(r_id, u_id) VALUES((@roomID), (SELECT u_id FROM Rooms WHERE Rooms.r_id = @roomID))";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@roomID", r_id);
                cmd.ExecuteNonQuery();

                tr.Commit();
                Chatroom c = new Chatroom(owner, roomName);
                c.addMember(owner);
                return c;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in creating room: " + ex.Message);
                try
                {
                    tr.Rollback();

                }
                catch (MySqlException ex1) { Console.WriteLine("failure in rolling back room: " + ex1.Message); }


                return null;
            }
        }
        public Chatroom CreateRoom(string roomName, string owner, string[] Users)
        {
            Chatroom c = CreateRoom(roomName, owner);
            if (c == null)
                return null;
            foreach (string u in Users)
                c.addMember(u);
            return c;
        }

        public int HasAccessToRoom(string chatroom, string user)
        {
            MySqlTransaction tr = null;
            try
            {
                tr = conn.BeginTransaction();

                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.Transaction = tr;

                cmd.CommandText = "SELECT EXISTS(SELECT r_id, u_id FROM R_U WHERE R_U.r_id = (SELECT Rooms.r_id FROM Rooms WHERE Rooms.r_name = @roomName) AND R_U.u_id = (SELECT Users.u_id FROM Users WHERE Users.u_name = @userName))";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@roomName", chatroom);
                cmd.Parameters.AddWithValue("@userName", user);
                rdr = cmd.ExecuteReader();
                rdr.Read();
                int result = rdr.GetInt32(0);
                rdr.Close();

                tr.Commit();
                rdr = null;
                return result;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in checking if user is part of room: " + ex.Message);
                try
                {
                    tr.Rollback();

                }
                catch (MySqlException ex1) { Console.WriteLine("failure in rolling back checking user is in room: " + ex1.Message); }
                if (rdr != null)
                {
                    rdr.Close();
                    rdr = null;
                }
                return -1;
            }
        }

        public int RoomAddUser(string chatroom, string user)
        {
            int res = HasAccessToRoom(chatroom, user);
            if (res == 0)
            {

                MySqlTransaction tr = null;
                try
                {
                    tr = conn.BeginTransaction();

                    MySqlCommand cmd = new MySqlCommand();
                    cmd.Connection = conn;
                    cmd.Transaction = tr;

                    cmd.CommandText = "INSERT INTO R_U(r_id, u_id) VALUES(((SELECT r_id FROM Rooms WHERE Rooms.r_name = @roomName)),(SELECT u_id FROM Users WHERE Users.u_name = @userName))";
                    cmd.Prepare();
                    cmd.Parameters.AddWithValue("@roomName", chatroom);
                    cmd.Parameters.AddWithValue("@userName", user);
                    cmd.ExecuteNonQuery();

                    tr.Commit();
                    return 1;
                }
                catch (MySqlException ex)
                {
                    Console.WriteLine("failure in adding user to room: " + ex.Message);
                    try
                    {
                        tr.Rollback();

                    }
                    catch (MySqlException ex1) { Console.WriteLine("failure in rolling back adding user to room: " + ex1.Message); }
                    return -1;
                }
            }
            return 0;
        }

        public int RoomRemoveUser(string chatroom, string user)
        {
            int res = HasAccessToRoom(chatroom, user);
            if (res == 1)
            {

                MySqlTransaction tr = null;
                try
                {
                    tr = conn.BeginTransaction();

                    MySqlCommand cmd = new MySqlCommand();
                    cmd.Connection = conn;
                    cmd.Transaction = tr;

                    cmd.CommandText = "DELETE FROM R_U WHERE R_U.r_id = (SELECT Rooms.r_id FROM Rooms WHERE Rooms.r_name = @roomName) AND R_U.u_id = (SELECT Users.u_id FROM Users WHERE Users.u_name = @userName)";
                    cmd.Prepare();
                    cmd.Parameters.AddWithValue("@roomName", chatroom);
                    cmd.Parameters.AddWithValue("@userName", user);
                    cmd.ExecuteNonQuery();

                    tr.Commit();
                    Console.WriteLine("Done");
                    return 1;
                }
                catch (MySqlException ex)
                {
                    Console.WriteLine("failure in removing user from room: " + ex.Message);
                    try
                    {
                        tr.Rollback();

                    }
                    catch (MySqlException ex1) { Console.WriteLine("failure in rolling back removing user from room: " + ex1.Message); }
                    return -1;
                }
            }
            return 0;
        }

        public List<Chatroom> GetChatroomList()
        {
            try
            {
                List<Chatroom> chatrooms = new List<Chatroom>();
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT r_name, u_name FROM Rooms JOIN Users ON Rooms.u_id = Users.u_id";
                rdr = cmd.ExecuteReader();

                MySqlCommand cmd2 = new MySqlCommand();
                
                while (rdr.Read())
                {
                    chatrooms.Add(new Chatroom(rdr.GetString(1), rdr.GetString(0)));
                }
                rdr.Close();
                rdr = null;

                foreach(Chatroom chatters in chatrooms)
                {
                    cmd = new MySqlCommand();
                    cmd.Connection = conn;
                    cmd.CommandText = "SELECT u_name FROM R_U JOIN Users ON Users.u_id = R_U.u_id AND R_U.r_id = (SELECT r_id FROM Rooms WHERE r_name = @roomName)";
                    cmd.Prepare();
                    cmd.Parameters.AddWithValue("@roomName", chatters.roomName);
                    rdr = cmd.ExecuteReader();

                    while(rdr.Read())
                    {
                        chatters.addMember(rdr.GetString(0));
                    }
                    rdr.Close();
                }

                return chatrooms;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in getting chatrooms: " + ex.Message);
                if (rdr != null) rdr.Close();
                return null;
            }
        }

        public List<Chatroom> GetUserChatroomAccessList(string userName)
        {
            try
            {
                List<Chatroom> chatrooms = new List<Chatroom>();
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT r_name, u_name FROM R_U JOIN Rooms ON R_U.r_id = Rooms.r_id JOIN Users ON Users.u_id = R_U.u_id AND Users.u_name = @userName";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@userName", userName);
                rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    chatrooms.Add(new Chatroom(rdr.GetString(1), rdr.GetString(0)));
                }
                rdr.Close();
                rdr = null;


                return chatrooms;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in getting chatroom access: " + ex.Message);
                if (rdr != null) rdr.Close();
                return null;
            }
        }

        public List<Chatroom> GetChatroomsOwned(string owner)
        {
            try
            {
                List<Chatroom> chatrooms = new List<Chatroom>();
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT r_name, u_name FROM Rooms JOIN Users ON Rooms.u_id = Users.u_id AND Users.u_name = @userName";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@userName", owner);
                rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    chatrooms.Add(new Chatroom(rdr.GetString(1), rdr.GetString(0)));
                }
                rdr.Close();
                rdr = null;


                return chatrooms;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in getting owned chatrooms: " + ex.Message);
                if (rdr != null) rdr.Close();
                return null;
            }
        }

        public string GetOwnerOfChatroom(string roomName)
        {
            try
            {
                string result = null;
                MySqlCommand cmd = new MySqlCommand();
                cmd.Connection = conn;
                cmd.CommandText = "SELECT u_name FROM Rooms JOIN Users ON Rooms.u_id = Users.u_id AND Rooms.r_name = @roomName";
                cmd.Prepare();
                cmd.Parameters.AddWithValue("@roomName", roomName);
                rdr = cmd.ExecuteReader();
                if (rdr.Read())
                {
                    result = rdr.GetString(0);
                }
                rdr.Close();
                rdr = null;


                return result;
            }
            catch (MySqlException ex)
            {
                Console.WriteLine("failure in getting owner of chatroom: " + ex.Message);
                if (rdr != null) rdr.Close();
                return null;
            }
        }
    }
}
