﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using DataBaseTest;
using AwesomeAudio;

namespace AudioChatServer
{
    public class InteractionManager
    {
        UdpServer _server;
        Stopwatch tokenTimer;

        public InteractionManager(UdpServer server)
        {
            _server = server;
            tokenTimer = new Stopwatch();

            //_server.addCallback(Protocol.CS_AUDIOSETUP, onAudioSetup);
            //_server.addCallback(Protocol.CS_AUDIOCHUNK, onAudioChunk);
            _server.addCallback(Protocol.CS_TOKENHOLD, onTokenHold);
            _server.addCallback(Protocol.CS_TOKENREQUEST, onTokenRequest);
            _server.addCallback(Protocol.CS_TOKENRELEASE, onTokenRelease);
            _server.addCallback(Protocol.CS_USERTXT, onUserTxt);
        }

        /*
        private int onAudioSetup(IPAddress ip, byte[] bytes)
        {
            MessageBox.Show("audio setup not fully implemented");
            try
            {
                if (_server.isIpLoggedIn(ip))
                {
                    User user = _server.userWithIP(ip);
                    if (user.activeRoom != null && user.hasToken)
                    {
                        byte[] msg = new byte[2];
                        msg[0] = 0x01;
                        msg[1] = Protocol.SC_AUDIOSETUP;

                        // Need to know what's in the message (sample rate, chunk size, etc)

                        List<string> roomMembers = user.activeRoom.members;

                        foreach (string u in roomMembers)
                        {
                            if (u != user.username)
                            {
                                _server.sendMessage(msg, _server.ipOfUser(u));
                            }
                        }

                        return Protocol.SC_AUDIOSETUP_SUCCESS;
                    }

                    user.hasToken = false;
                    return Protocol.SC_AUDIOSETUP_FAIL;
                }
                else
                {
                    return Protocol.SC_AUDIOSETUP_FAIL;
                }
            }
            catch (Exception e)
            {
                _server.userWithIP(ip).hasToken = false;

                Console.WriteLine("InteractionManager.onAudioSetup() failed: " + e.Message);
                return Protocol.SC_AUDIOSETUP_FAIL;
            }
        }

        private int onAudioChunk(IPAddress ip, byte[] bytes)
        {
            MessageBox.Show("audio chunk not fully implemented");
            try
            {
                if (_server.isIpLoggedIn(ip))
                {
                    User user = _server.userWithIP(ip);
                    if (user.hasToken && user.activeRoom != null)
                    {
                        byte[] msg = new byte[2];
                        msg[0] = 0x01;
                        msg[1] = Protocol.SC_AUDIOCHUNK;

                        // Need to know what's in the message (sample rate, chunk size, etc)

                        List<string> roomMembers = user.activeRoom.members;

                        foreach (string u in roomMembers)
                        {
                            if (u != user.username)
                            {
                                _server.sendMessage(msg, _server.ipOfUser(u));
                            }
                        }

                        return Protocol.SC_SUCCESS;
                    }

                    user.hasToken = false;
                    return Protocol.SC_FAILURE;
                }
                else
                {
                    return Protocol.SC_FAILURE;
                }
            }
            catch (Exception e)
            {
                _server.userWithIP(ip).hasToken = false;

                Console.WriteLine("InteractionManager.onAudioChunk() failed: " + e.Message);
                return Protocol.SC_FAILURE;
            }
        }
        */

        private int onTokenHold(IPAddress ip, byte[] bytes)
        {
            try
            {
                if (_server.isIpLoggedIn(ip))
                {
                    User user = _server.userWithIP(ip);

                    if (!user.hasToken)
                    {
                        return Protocol.SC_TOKENHOLD_FAIL;
                    }

                    tokenTimer.Restart();
                    return Protocol.SC_TOKENHOLD_SUCCESS;
                }
                else
                {
                    return Protocol.SC_TOKENHOLD_FAIL;
                }
            }
            catch (Exception e)
            {
                _server.userWithIP(ip).hasToken = false;

                Console.WriteLine(e.Message);
                return Protocol.SC_TOKENHOLD_FAIL;
            }
        }

        private int onTokenRequest(IPAddress ip, byte[] bytes)
        {
            try
            {
                if (_server.isIpLoggedIn(ip))
                {
                    User user = _server.userWithIP(ip);

                    if (user.activeRoom != null)
                    {
                        if (tokenTimer.ElapsedMilliseconds >= 500)
                        {
                            user.activeRoom.userWithToken = "";
                            tokenTimer.Reset();
                        }

                        if(user.activeRoom.userWithToken == "")
                        {
                            user.hasToken = true;
                            tokenTimer.Start();

                            List<byte> ipListMsg = new List<byte>();
                            ipListMsg.Add(0x01);
                            ipListMsg.Add(Protocol.SC_TOKENREQUEST_SUCCESS);
                            ipListMsg.Add((byte)(user.activeRoom.activeMembers.Count - 1));

                            foreach (string u in user.activeRoom.activeMembers)
                            {
                                if (u != user.username)
                                {
                                    IPAddress userIp = _server.ipOfUser(u);

                                    if (userIp != null)
                                    {
                                        byte[] ipBytes = new byte[Protocol.SIZE_IPTXT];
                                        ASCIIEncoding.ASCII.GetBytes(userIp.ToString()).ToList().CopyTo(ipBytes);
                                        ipListMsg.AddRange(ipBytes.ToList());

                                        // Send audio begin message to each client
                                        byte[] audioBeginMsg = { 0x01, Protocol.SC_AUDIOBEGIN };
                                        _server.sendMessage(audioBeginMsg, userIp);
                                    }
                                }
                            }

                            _server.sendMessage(ipListMsg.ToArray(), ip);

                            return Protocol.CS_SUCCESS;
                        }
                    }

                    return Protocol.SC_TOKENREQUEST_FAIL;
                }
                else
                {
                    return Protocol.SC_TOKENREQUEST_FAIL;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                return Protocol.SC_TOKENREQUEST_FAIL;
            }
        }

        private int onTokenRelease(IPAddress ip, byte[] bytes)
        {
            try
            {
                if (_server.isIpLoggedIn(ip))
                {
                    User user = _server.userWithIP(ip);

                    if (user.activeRoom != null)
                    {
                        user.activeRoom.userWithToken = "";
                        tokenTimer.Reset();

                        foreach (string u in user.activeRoom.activeMembers)
                        {
                            if (u != user.username)
                            {
                                IPAddress userIp = _server.ipOfUser(u);

                                if (userIp != null)
                                {
                                    // Send audio end message to each client
                                    byte[] audioBeginMsg = { 0x01, Protocol.SC_AUDIOEND };
                                    _server.sendMessage(audioBeginMsg, userIp);
                                }
                            }
                        }
                        return Protocol.SC_SUCCESS;
                    }

                    return Protocol.SC_FAILURE;
                }
                else
                {
                    return Protocol.SC_FAILURE;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                return Protocol.SC_FAILURE;
            }
        }

        private int onUserTxt(IPAddress ip, byte[] bytes)
        {
            try
            {
                if (_server.isIpLoggedIn(ip))
                {
                    byte[] msgToClients = new byte[Protocol.SIZE_USERTXT + 3];
                    msgToClients = (byte[])bytes.Clone();
                    msgToClients[1] = Protocol.SC_USERTXT;

                    if (bytes[2] == 0x01)
                    {
                        if (_server.userWithIP(ip).activeRoom != null)
                        {
                            // send message to all users in private room
                            foreach (string u in _server.userWithIP(ip).activeRoom.activeMembers)
                            {
                                _server.sendMessage(msgToClients, _server.userWithName(u).ipAddress, (int)(3 + Protocol.SIZE_USERTXT));
                            }
                        }
                        else
                        {
                            return Protocol.SC_FAILURE;
                        }
                    }
                    else
                    {
                        // send message to all users in public room
                        foreach (User u in _server.ConnectedUsers.Values)
                        {
                            _server.sendMessage(msgToClients, u.ipAddress, (int)(3 + Protocol.SIZE_USERTXT));
                        }
                    }

                    return Protocol.SC_SUCCESS;
                }
                else
                {
                    Console.WriteLine("txt disperse failed because " + ip.ToString() + " is not logged in");
                    return Protocol.SC_FAILURE;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("user txt disperse failed : " + e.Message);
                return Protocol.SC_FAILURE;
            }
        }
    }
}
