﻿/*
 * Chris Logsdon
 * CP 252 Final Project
 * Audio Chat Server
 * 
 * Partners:
 * Zack Wheeler (Database, Documentation, Overseer)
 * Gina MacIntyre (Client)
 * 
 * Credits:
 * 
 * List Box Item color change:
 * http://stackoverflow.com/questions/6896151/c-sharp-change-listbox-items-color
 * http://stackoverflow.com/questions/3663704/how-to-change-listbox-selection-background-color
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace AudioChatServer
{
    public partial class MainForm : Form
    {
        public UdpServer server;
        private ServerConnectionForm serverSetupForm;
        private LogWindow log;
        public delegate void UpdateDelegate();


        public MainForm()
        {
            InitializeComponent();

            server = new UdpServer(onMessageReceive, UpdateForm);
            serverSetupForm = new ServerConnectionForm(this);
            log = new LogWindow(this);
        }

        private void miSetupConnection_Click(object sender, EventArgs e)
        {
            serverSetupForm.ShowDialog();
        }

        public void onMessageReceive(string msg)
        {
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            updateControls(false);
            gbCRdetails.Enabled = false;
            gbCUuserInfo.Enabled = false;
        }

        public void onServerStarted()
        {
            updateControls(true);
            UpdateForm();
        }

        public void onServerStopped()
        {
            updateControls(false);
        }

        private void MainForm_Shown(object sender, EventArgs e)
        {
            serverSetupForm.ShowDialog();
        }

        private void updateControls(bool listening)
        {
            gbCUuserList.Enabled = listening;
            gbCRrooms.Enabled = listening;

            lbCUusersOnline.ClearSelected();
        }

        public void UpdateForm()
        {
            if (this.InvokeRequired)
            {
                UpdateDelegate d = new UpdateDelegate(UpdateForm);
                this.Invoke(d);
            }
            else
            {
                // Update users list box in the Users tab
                int selectedIndex = lbCUusersOnline.SelectedIndex;
                lbCUusersOnline.DataSource = server.UsersInDatabase;
                lbCUusersOnline.Update();

                if (selectedIndex >= 0)
                {
                    lbCUusersOnline.SetSelected(selectedIndex, true); // Maintain the user's selection
                }

                // Update chat rooms list in Chatrooms tab
                selectedIndex = lbCRrooms.SelectedIndex;
                lbCRrooms.DataSource = server.RoomsInDatabase;
                lbCRrooms.Update();

                if (selectedIndex >= 0)
                {
                    lbCRrooms.SetSelected(selectedIndex, true); // Maintain user's selection
                }

                Stopwatch timer = new Stopwatch();

                // print all the messages in the log queue
                timer.Start();
                while (server.logMessages.Count > 0)
                {
                    // Don't let this loop run forever
                    if (timer.Elapsed.Seconds >= 3)
                    {
                        timer.Reset();
                        break;
                    }

                    log.logMessage(server.logMessages.Dequeue());
                }
            }
        }

        private void lbCRrooms_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbCRrooms.SelectedIndex >= 0 && lbCRrooms.SelectedIndex < lbCRrooms.Items.Count)
            {
                gbCRdetails.Enabled = true;

                labelCRroomName.Text = (lbCRrooms.SelectedItem as Chatroom).roomName;
                labelCRownerUser.Text = (lbCRrooms.SelectedItem as Chatroom).owner;

                lbCRmembers.DataSource = (lbCRrooms.SelectedItem as Chatroom).members;
                lbCRmembers.Update();
            }
        }

        private void lbCUconnectedUsers_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbCUusersOnline.SelectedIndex >= 0 && lbCUusersOnline.SelectedIndex < lbCUusersOnline.Items.Count)
            {
                gbCUuserInfo.Enabled = true;

                labelCUusernameData.Text = (lbCUusersOnline.SelectedItem as User).username;

                User userItem = lbCUusersOnline.Items[lbCUusersOnline.SelectedIndex] as User;
                if (userItem != null)
                {
                    if (server.isUserLoggedIn(userItem))
                    {
                        labelCUipData.Text = server.ipOfUser(userItem).ToString();
                    }
                    else
                    {
                        labelCUipData.Text = "N/A";
                    }
                }

                lbCUchatRooms.DataSource = server.roomMembership(userItem.username);
                lbCUchatRooms.Update();
            }
        }

        private void lbCUconnectedUsers_DrawItem(object sender, DrawItemEventArgs e)
        {
            if (e.Index >= 0)
            {
                Color txtColor = Color.Gray;
                Color highlightColor = Color.LightBlue;

                User userItem = lbCUusersOnline.Items[e.Index] as User;
                if (userItem != null)
                {
                    // If the user is logged in...
                    if (server.isUserLoggedIn(userItem))
                    {
                        txtColor = Color.Black; // Draw name in black
                        highlightColor = Color.FromArgb(220, 255, 220); // highlight in green
                    }
                    else // if not logged in...
                    {
                        txtColor = Color.Gray; // Draw name in gray
                        highlightColor = Color.FromArgb(255, 220, 220); // highlight in red
                    }
                }

                // This allows us to change the selection rectangle of the selected item
                if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                {
                    e = new DrawItemEventArgs(e.Graphics, e.Font, e.Bounds, e.Index,
                                              e.State ^ DrawItemState.Selected,
                                              e.ForeColor, highlightColor);
                }

                // Draw order matters
                e.DrawBackground();

                if (userItem != null)
                {
                    e.Graphics.DrawString(
                        userItem.username,                          // Draw the username text
                        lbCUusersOnline.Font,                    // Default font
                        new SolidBrush(txtColor),                   // Font color
                        0, e.Index * lbCUusersOnline.ItemHeight);// Location of the text
                }

                e.DrawFocusRectangle();
            }
        }

        private void miShowLog_Click(object sender, EventArgs e)
        {
            if (log.Visible)
            {
                log.BringToFront();
            }
            else
            {
                log.Visible = true;
            }
        }

        public void onKeyPress(object sender, KeyPressEventArgs e)
        {
            switch (char.ToUpper(e.KeyChar))
            {
                case 'L':
                    log.Visible = !log.Visible;
                    break;
                case 'S':
                    serverSetupForm.ShowDialog();
                    break;
                case 'U':
                    tabControl.SelectTab(tabUsers.Name);
                    break;
                case 'R':
                    tabControl.SelectTab(tabChatRooms.Name);
                    break;
                default:
                    break;
            }
        }

        private void howToToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "To start or stop the server, simply click on \"Server\" -> \"Manage Connection\".\n" +
                "In the window that pops up, click on the \"Start Server\" or \"Stop Server\" buttons.\n" +
                "\n" +
                "If you use a port other than 4321, be sure to tell your clients what the new port is!",
                "How-To");

        }

        private void shortcutsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "L = Hide/Show Log Window\n" +
                "U = Open \"Users\" tab\n" +
                "R = Open \"Chat Rooms\" tab\n" +
                "S = Open \"Server Connection\" window",
                "Shortcuts");
        }
    }
}
