﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AudioChatServer
{
    public partial class ServerConnectionForm : Form
    {
        MainForm parent;

        public ServerConnectionForm(MainForm parent)
        {
            InitializeComponent();

            this.parent = parent;
            txtIPAddress.Text = parent.server.ipAddress;
        }

        private void bStartServer_Click(object sender, EventArgs e)
        {
            if (txtIPAddress.Text == "")
            {
                MessageBox.Show("Enter a valid IP Address");
                return;
            }

            if (txtPort.Text == "")
            {
                MessageBox.Show("Enter a valid port number");
                return;
            }

            if (parent.server.setIPAddress(txtIPAddress.Text) == null)
            {
                MessageBox.Show(txtIPAddress.Text + " is not a valid IP Address.");
                return;
            }

            int port = 0;
            try
            {
                port = Convert.ToInt32(txtPort.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(txtPort.Text + " is not a valid port number.");
                Console.WriteLine(ex.Message);
                return;
            }

            try
            {
                labelStatus.Text = "Setting up connection...";

                parent.server.startListening(port);

                labelStatus.Text = "Connected. Now Listening...";
                updateControls(parent.server.isListening);
                parent.onServerStarted();
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    "Error in establishing connection using\n" +
                    "IP: " + parent.server.ipAddress + "\n" +
                    "Port: " + parent.server.portNumberListen);

                labelStatus.Text = "Connection failed";

                Console.WriteLine(ex.Message);
            }

        }

        private void bStopServer_Click(object sender, EventArgs e)
        {
            labelStatus.Text = "Closing connection...";

            parent.server.stopListening();

            labelStatus.Text = "Connection closed";
            updateControls(parent.server.isListening);

            parent.onServerStopped();
        }

        private void ServerSetupForm_Load(object sender, EventArgs e)
        {
            if (parent.server == null) return;

            bStartServer.Enabled = !parent.server.isListening;
            bStopServer.Enabled = parent.server.isListening;
        }

        private void bGetIP_Click(object sender, EventArgs e)
        {
            txtIPAddress.Text = parent.server.findIPAddress().ToString();
        }

        private void updateControls(bool serverListening)
        {
            bStartServer.Enabled = !serverListening;
            bStopServer.Enabled = serverListening;
            txtIPAddress.Enabled = !serverListening;
            txtPort.Enabled = !serverListening;
            bGetIP.Enabled = !serverListening;
        }

        private void ServerSetupForm_KeyPress(object sender, KeyPressEventArgs e)
        {
            parent.onKeyPress(sender, e);
        }
    }
}
